@@ -1,19 +1,7 @@
 /* This file is part of the KDE libraries
-   Copyright (C) 2011 Mario Bensi <mbensi@ipsquad.net>
+   SPDX-FileCopyrightText: 2011 Mario Bensi <mbensi@ipsquad.net>
 
-   This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
-   License version 2 as published by the Free Software Foundation.
-
-   This library is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
-
-   You should have received a copy of the GNU Library General Public License
-   along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301, USA.
+   SPDX-License-Identifier: LGPL-2.0-only
 */
 
 #include "k7zip.h"
@@ -1114,7 +1102,7 @@ static uint toTimeT(const long long liTime)
 
     QDateTime t(QDate(year, month, day), QTime(hour, minute, second));
     t.setTimeSpec(Qt::UTC);
-    return t.toTime_t();
+    return t.toSecsSinceEpoch();
 }
 
 long long rtlSecondsSince1970ToSpecTime(quint32 seconds)
@@ -1446,6 +1434,15 @@ static QByteArray decodeBCJ2(const QByteArray &mainStream, const QByteArray &cal
 
         unsigned index = getIndex(prevByte, b);
         if (statusDecoder[index].decode(&rangeDecoder) == 1) {
+            if (b == 0xE8) {
+                if (callStreamPos + 4 > callStream.size()) {
+                    return QByteArray();
+                }
+            } else {
+                if (jumpStreamPos + 4 > jumpStream.size()) {
+                    return QByteArray();
+                }
+            }
             quint32 src = 0;
             for (int i = 0; i < 4; i++) {
                 unsigned char b0;
@@ -1683,6 +1680,10 @@ QByteArray K7Zip::K7ZipPrivate::readAndDecodePackedStreams(bool readMainStreamIn
         inflatedDatas.clear();
 
         if (folder->unpackCRCDefined) {
+            if ((size_t)inflated.size() < unpackSize) {
+                qCDebug(KArchiveLog) << "wrong crc size data";
+                return QByteArray();
+            }
             quint32 crc = crc32(0, (Bytef *)(inflated.data()), unpackSize);
             if (crc != folder->unpackCRC) {
                 qCDebug(KArchiveLog) << "wrong crc";
@@ -1710,7 +1711,7 @@ void K7Zip::K7ZipPrivate::createItemsFromEntities(const KArchiveDirectory *dir,
 
         fileInfo->path = path + entry->name();
         mTimesDefined.append(true);
-        mTimes.append(rtlSecondsSince1970ToSpecTime(entry->date().toTime_t()));
+        mTimes.append(rtlSecondsSince1970ToSpecTime(entry->date().toSecsSinceEpoch()));
 
         if (entry->isFile()) {
             const K7ZipFileEntry *fileEntry = static_cast<const K7ZipFileEntry *>(entry);
