@@ -1,26 +1,15 @@
 /* This file is part of the KDE libraries
-   Copyright (C) 2000 David Faure <faure@kde.org>
-   Copyright (C) 2011 Mario Bensi <mbensi@ipsquad.net>
+   SPDX-FileCopyrightText: 2000 David Faure <faure@kde.org>
+   SPDX-FileCopyrightText: 2011 Mario Bensi <mbensi@ipsquad.net>
 
-   This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
-   License version 2 as published by the Free Software Foundation.
-
-   This library is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
-
-   You should have received a copy of the GNU Library General Public License
-   along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301, USA.
+   SPDX-License-Identifier: LGPL-2.0-only
 */
 #ifndef __kcompressiondevice_h
 #define __kcompressiondevice_h
 
 #include <karchive_export.h>
 #include <QIODevice>
+#include <QFileDevice>
 #include <QString>
 #include <QMetaType>
 class KCompressionDevicePrivate;
@@ -36,8 +25,9 @@ class KFilterBase;
  * Use this class to read/write compressed files.
  */
 
-class KARCHIVE_EXPORT KCompressionDevice : public QIODevice
+class KARCHIVE_EXPORT KCompressionDevice : public QIODevice // KF6 TODO: consider inheriting from QFileDevice, so apps can use error() generically ?
 {
+    Q_OBJECT
 public:
     enum CompressionType {
         GZip,
@@ -113,6 +103,14 @@ public:
      */
     static KFilterBase *filterForCompressionType(CompressionType type);
 
+    /**
+     * Returns the error code from the last failing operation.
+     * This is especially useful after calling close(), which unfortunately returns void
+     * (see https://bugreports.qt.io/browse/QTBUG-70033), to see if the flushing done by close
+     * was able to write all the data to disk.
+     */
+    QFileDevice::FileError error() const;
+
 protected:
     friend class K7Zip;
 
@@ -121,6 +119,7 @@ protected:
 
     KFilterBase *filterBase();
 private:
+    friend KCompressionDevicePrivate;
     KCompressionDevicePrivate *const d;
 };
 
