@@ -1,33 +1,22 @@
 /* This file is part of the KDE libraries
-   Copyright (C) 2000 David Faure <faure@kde.org>
-   Copyright (C) 2003 Leo Savernik <l.savernik@aon.at>
+   SPDX-FileCopyrightText: 2000 David Faure <faure@kde.org>
+   SPDX-FileCopyrightText: 2003 Leo Savernik <l.savernik@aon.at>
 
-   This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
-   License version 2 as published by the Free Software Foundation.
-
-   This library is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
-
-   You should have received a copy of the GNU Library General Public License
-   along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301, USA.
+   SPDX-License-Identifier: LGPL-2.0-only
 */
 
 #include "ktar.h"
 #include "karchive_p.h"
+#include "loggingcategory.h"
 
 #include <stdlib.h> // strtol
 #include <assert.h>
 
-#include <QtCore/QDebug>
-#include <QtCore/QDir>
-#include <QtCore/QFile>
+#include <QDebug>
+#include <QDir>
+#include <QFile>
 #include <qmimedatabase.h>
-#include <QtCore/QTemporaryFile>
+#include <QTemporaryFile>
 
 #include <kfilterdev.h>
 #include <kfilterbase.h>
@@ -42,14 +31,14 @@ static const char application_bzip[] = "application/x-bzip";
 static const char application_lzma[] = "application/x-lzma";
 static const char application_xz[] = "application/x-xz";
 
-class KTar::KTarPrivate
+class Q_DECL_HIDDEN KTar::KTarPrivate
 {
 public:
     KTarPrivate(KTar *parent)
         : q(parent)
         , tarEnd(0)
-        , tmpFile(0)
-        , compressionDevice(Q_NULLPTR)
+        , tmpFile(nullptr)
+        , compressionDevice(nullptr)
     {
     }
 
@@ -108,7 +97,7 @@ bool KTar::createDevice(QIODevice::OpenMode mode)
             mime = db.mimeTypeForFile(fileName(), QMimeDatabase::MatchExtension);
         }
 
-        //qDebug() << mode << mime->name();
+        //qCDebug(KArchiveLog) << mode << mime->name();
 
         if (mime.inherits(QStringLiteral("application/x-compressed-tar")) || mime.inherits(QString::fromLatin1(application_gzip))) {
             // gzipped tar file (with possibly invalid file name), ask for gzip filter
@@ -133,7 +122,7 @@ bool KTar::createDevice(QIODevice::OpenMode mode)
         }
         if (!d->mimetype.isEmpty()) {
             // Create a compression filter on top of the QSaveFile device that KArchive created.
-            //qDebug() << "creating KFilterDev for" << d->mimetype;
+            //qCDebug(KArchiveLog) << "creating KFilterDev for" << d->mimetype;
             KCompressionDevice::CompressionType type = KFilterDev::compressionTypeForMimeType(d->mimetype);
             d->compressionDevice = new KCompressionDevice(device(), false, type);
             setDevice(d->compressionDevice);
@@ -154,7 +143,7 @@ bool KTar::createDevice(QIODevice::OpenMode mode)
         d->tmpFile = new QTemporaryFile();
         d->tmpFile->setFileTemplate(QDir::tempPath() + QStringLiteral("/") + QLatin1String("ktar-XXXXXX.tar"));
         d->tmpFile->open();
-        //qDebug() << "creating tempfile:" << d->tmpFile->fileName();
+        //qCDebug(KArchiveLog) << "creating tempfile:" << d->tmpFile->fileName();
 
         setDevice(d->tmpFile);
         return true;
@@ -176,7 +165,7 @@ KTar::~KTar()
 void KTar::setOrigFileName(const QByteArray &fileName)
 {
     if (!isOpen() || !(mode() & QIODevice::WriteOnly)) {
-        //qWarning() << "KTar::setOrigFileName: File must be opened for writing first.\n";
+        //qCWarning(KArchiveLog) << "KTar::setOrigFileName: File must be opened for writing first.\n";
         return;
     }
     d->origFileName = fileName;
@@ -195,12 +184,12 @@ qint64 KTar::KTarPrivate::readRawHeader(char *buffer)
 
             int check = 0;
             for (uint j = 0; j < 0x200; ++j) {
-                check += buffer[j];
+                check += static_cast<unsigned char>(buffer[j]);
             }
 
             // adjust checksum to count the checksum fields as blanks
             for (uint j = 0; j < 8 /*size of the checksum field including the \0 and the space*/; j++) {
-                check -= buffer[148 + j];
+                check -= static_cast<unsigned char>(buffer[148 + j]);
             }
             check += 8 * ' ';
 
@@ -212,7 +201,7 @@ qint64 KTar::KTarPrivate::readRawHeader(char *buffer)
             if (strncmp(buffer + 148 + 6 - s.length(), s.data(), s.length())
                 && strncmp(buffer + 148 + 7 - s.length(), s.data(), s.length())
                 && strncmp(buffer + 148 + 8 - s.length(), s.data(), s.length())) {
-                /*qWarning() << "KTar: invalid TAR file. Header is:" << QByteArray( buffer+257, 5 )
+                /*qCWarning(KArchiveLog) << "KTar: invalid TAR file. Header is:" << QByteArray( buffer+257, 5 )
                                << "instead of ustar. Reading from wrong pos in file?"
                                << "checksum=" << QByteArray( buffer + 148 + 6 - s.length(), s.length() );*/
                 return -1;
@@ -230,13 +219,21 @@ qint64 KTar::KTarPrivate::readRawHeader(char *buffer)
 bool KTar::KTarPrivate::readLonglink(char *buffer, QByteArray &longlink)
 {
     qint64 n = 0;
-    //qDebug() << "reading longlink from pos " << q->device()->pos();
+    //qCDebug(KArchiveLog) << "reading longlink from pos " << q->device()->pos();
     QIODevice *dev = q->device();
     // read size of longlink from size field in header
     // size is in bytes including the trailing null (which we ignore)
-    qint64 size = QByteArray(buffer + 0x7c, 12).trimmed().toLongLong(0, 8 /*octal*/);
+    qint64 size = QByteArray(buffer + 0x7c, 12).trimmed().toLongLong(nullptr, 8 /*octal*/);
 
     size--;    // ignore trailing null
+    if (size > std::numeric_limits<int>::max() - 32) { // QByteArray can't really be INT_MAX big, it's max size is something between INT_MAX - 32 and INT_MAX depending the platform so just be safe
+        qCWarning(KArchiveLog) << "Failed to allocate memory for longlink of size" << size;
+        return false;
+    }
+    if (size < 0) {
+        qCWarning(KArchiveLog) << "Invalid longlink size" << size;
+        return false;
+    }
     longlink.resize(size);
     qint64 offset = 0;
     while (size > 0) {
@@ -273,15 +270,16 @@ qint64 KTar::KTarPrivate::readHeader(char *buffer, QString &name, QString &symli
         if (strcmp(buffer, "././@LongLink") == 0) {
             char typeflag = buffer[0x9c];
             QByteArray longlink;
-            readLonglink(buffer, longlink);
-            switch (typeflag) {
-            case 'L':
-                name = QFile::decodeName(longlink.constData());
-                break;
-            case 'K':
-                symlink = QFile::decodeName(longlink.constData());
-                break;
-            }/*end switch*/
+            if (readLonglink(buffer, longlink)) {
+                switch (typeflag) {
+                case 'L':
+                    name = QFile::decodeName(longlink.constData());
+                    break;
+                case 'K':
+                    symlink = QFile::decodeName(longlink.constData());
+                    break;
+                }/*end switch*/
+            }
         } else {
             break;
         }/*end if*/
@@ -313,7 +311,7 @@ bool KTar::KTarPrivate::fillTempFile(const QString &fileName)
         return true;
     }
 
-    //qDebug() << "filling tmpFile of mimetype" << mimetype;
+    //qCDebug(KArchiveLog) << "filling tmpFile of mimetype" << mimetype;
 
     KCompressionDevice::CompressionType compressionType = KFilterDev::compressionTypeForMimeType(mimetype);
     KCompressionDevice filterDev(fileName, compressionType);
@@ -325,15 +323,20 @@ bool KTar::KTarPrivate::fillTempFile(const QString &fileName)
     QByteArray buffer;
     buffer.resize(8 * 1024);
     if (! filterDev.open(QIODevice::ReadOnly)) {
+        q->setErrorString(
+            tr("File %1 does not exist")
+            .arg(fileName));
         return false;
     }
     qint64 len = -1;
     while (!filterDev.atEnd() && len != 0) {
         len = filterDev.read(buffer.data(), buffer.size());
         if (len < 0) {   // corrupted archive
+            q->setErrorString(tr("Archive %1 is corrupt").arg(fileName));
             return false;
         }
         if (file->write(buffer.data(), len) != len) {   // disk full
+            q->setErrorString(tr("Disk full"));
             return false;
         }
     }
@@ -344,7 +347,7 @@ bool KTar::KTarPrivate::fillTempFile(const QString &fileName)
     Q_ASSERT(file->isOpen());
     Q_ASSERT(file->openMode() & QIODevice::ReadOnly);
 
-    //qDebug() << "filling tmpFile finished.";
+    //qCDebug(KArchiveLog) << "filling tmpFile finished.";
     return true;
 }
 
@@ -368,6 +371,8 @@ bool KTar::openArchive(QIODevice::OpenMode mode)
     QIODevice *dev = device();
 
     if (!dev) {
+        setErrorString(tr("Could not get underlying device"));
+        qCWarning(KArchiveLog) << "Could not get underlying device";
         return false;
     }
 
@@ -381,6 +386,7 @@ bool KTar::openArchive(QIODevice::OpenMode mode)
         // Read header
         qint64 n = d->readHeader(buffer, name, symlink);
         if (n < 0) {
+            setErrorString(tr("Could not read tar header"));
             return false;
         }
         if (n == 0x200) {
@@ -412,8 +418,13 @@ bool KTar::openArchive(QIODevice::OpenMode mode)
             int access = strtol(p, &dummy, 8);
 
             // read user and group
-            QString user = QString::fromLocal8Bit(buffer + 0x109);
-            QString group = QString::fromLocal8Bit(buffer + 0x129);
+            const int maxUserGroupLength = 32;
+            const char *userStart = buffer + 0x109;
+            const int userLen = qstrnlen(userStart, maxUserGroupLength);
+            const QString user = QString::fromLocal8Bit(userStart, userLen);
+            const char *groupStart = buffer + 0x129;
+            const int groupLen = qstrnlen(groupStart, maxUserGroupLength);
+            const QString group = QString::fromLocal8Bit(groupStart, groupLen);
 
             // read time
             buffer[0x93] = 0;
@@ -439,7 +450,7 @@ bool KTar::openArchive(QIODevice::OpenMode mode)
                 isdir = false;
                 isDumpDir = true;
             }
-            //qDebug() << nm << "isdir=" << isdir << "pos=" << dev->pos() << "typeflag=" << typeflag << " islink=" << ( typeflag == '1' || typeflag == '2' );
+            //qCDebug(KArchiveLog) << nm << "isdir=" << isdir << "pos=" << dev->pos() << "typeflag=" << typeflag << " islink=" << ( typeflag == '1' || typeflag == '2' );
 
             if (typeflag == 'x' || typeflag == 'g') { // pax extended header, or pax global extended header
                 // Skip it for now. TODO: implement reading of extended header, as per http://pubs.opengroup.org/onlinepubs/009695399/utilities/pax.html
@@ -453,27 +464,27 @@ bool KTar::openArchive(QIODevice::OpenMode mode)
 
             KArchiveEntry *e;
             if (isdir) {
-                //qDebug() << "directory" << nm;
+                //qCDebug(KArchiveLog) << "directory" << nm;
                 e = new KArchiveDirectory(this, nm, access, KArchivePrivate::time_tToDateTime(time), user, group, symlink);
             } else {
                 // read size
                 QByteArray sizeBuffer(buffer + 0x7c, 12);
-                qint64 size = sizeBuffer.trimmed().toLongLong(0, 8 /*octal*/);
-                //qDebug() << "sizeBuffer='" << sizeBuffer << "' -> size=" << size;
+                qint64 size = sizeBuffer.trimmed().toLongLong(nullptr, 8 /*octal*/);
+                //qCDebug(KArchiveLog) << "sizeBuffer='" << sizeBuffer << "' -> size=" << size;
 
                 // for isDumpDir we will skip the additional info about that dirs contents
                 if (isDumpDir) {
-                    //qDebug() << nm << "isDumpDir";
+                    //qCDebug(KArchiveLog) << nm << "isDumpDir";
                     e = new KArchiveDirectory(this, nm, access, KArchivePrivate::time_tToDateTime(time), user, group, symlink);
                 } else {
 
                     // Let's hack around hard links. Our classes don't support that, so make them symlinks
                     if (typeflag == '1') {
-                        //qDebug() << "Hard link, setting size to 0 instead of" << size;
+                        //qCDebug(KArchiveLog) << "Hard link, setting size to 0 instead of" << size;
                         size = 0; // no contents
                     }
 
-                    //qDebug() << "file" << nm << "size=" << size;
+                    //qCDebug(KArchiveLog) << "file" << nm << "size=" << size;
                     e = new KArchiveFile(this, nm, access, KArchivePrivate::time_tToDateTime(time), user, group, symlink,
                                          dev->pos(), size);
                 }
@@ -481,17 +492,23 @@ bool KTar::openArchive(QIODevice::OpenMode mode)
                 // Skip contents + align bytes
                 qint64 rest = size % 0x200;
                 qint64 skip = size + (rest ? 0x200 - rest : 0);
-                //qDebug() << "pos()=" << dev->pos() << "rest=" << rest << "skipping" << skip;
+                //qCDebug(KArchiveLog) << "pos()=" << dev->pos() << "rest=" << rest << "skipping" << skip;
                 if (! dev->seek(dev->pos() + skip)) {
-                    //qWarning() << "skipping" << skip << "failed";
+                    //qCWarning(KArchiveLog) << "skipping" << skip << "failed";
                 }
             }
 
             if (pos == -1) {
                 if (nm == QLatin1String(".")) { // special case
-                    Q_ASSERT(isdir);
                     if (isdir) {
-                        setRootDir(static_cast<KArchiveDirectory *>(e));
+                        if (KArchivePrivate::hasRootDir(this)) {
+                            qWarning() << "Broken tar file has two root dir entries";
+                            delete e;
+                        } else {
+                            setRootDir(static_cast<KArchiveDirectory *>(e));
+                        }
+                    } else {
+                        delete e;
                     }
                 } else {
                     rootDir()->addEntry(e);
@@ -501,10 +518,15 @@ bool KTar::openArchive(QIODevice::OpenMode mode)
                 QString path = QDir::cleanPath(name.left(pos));
                 // Ensure container directory exists, create otherwise
                 KArchiveDirectory *d = findOrCreate(path);
-                d->addEntry(e);
+                if (d) {
+                    d->addEntry(e);
+                } else {
+                    delete e;
+                    return false;
+                }
             }
         } else {
-            //qDebug("Terminating. Read %d bytes, first one is %d", n, buffer[0]);
+            //qCDebug(KArchiveLog) << "Terminating. Read " << n << " bytes, first one is " << buffer[0];
             d->tarEnd = dev->pos() - n; // Remember end of archive
             ende = true;
         }
@@ -523,7 +545,7 @@ bool KTar::KTarPrivate::writeBackTempFile(const QString &fileName)
         return true;
     }
 
-    //qDebug() << "Write temporary file to compressed file" << fileName << mimetype;
+    //qCDebug(KArchiveLog) << "Write temporary file to compressed file" << fileName << mimetype;
 
     bool forced = false;
     if (QLatin1String(application_gzip) == mimetype || QLatin1String(application_bzip) == mimetype ||
@@ -539,6 +561,7 @@ bool KTar::KTarPrivate::writeBackTempFile(const QString &fileName)
     QFile *file = tmpFile;
     if (!dev.open(QIODevice::WriteOnly)) {
         file->close();
+        q->setErrorString(tr("Failed to write back temp file: %1").arg(dev.errorString()));
         return false;
     }
     if (forced) {
@@ -555,7 +578,7 @@ bool KTar::KTarPrivate::writeBackTempFile(const QString &fileName)
     file->close();
     dev.close();
 
-    //qDebug() << "Write temporary file to compressed file done.";
+    //qCDebug(KArchiveLog) << "Write temporary file to compressed file done.";
     return true;
 }
 
@@ -571,8 +594,8 @@ bool KTar::closeArchive()
     if (d->tmpFile && (mode() & QIODevice::WriteOnly)) {
         ok = d->writeBackTempFile(fileName());
         delete d->tmpFile;
-        d->tmpFile = 0;
-        setDevice(0);
+        d->tmpFile = nullptr;
+        setDevice(nullptr);
     }
 
     return ok;
@@ -591,7 +614,15 @@ bool KTar::doFinishWriting(qint64 size)
             buffer[i] = 0;
         }
         qint64 nwritten = device()->write(buffer, 0x200 - rest);
-        return nwritten == 0x200 - rest;
+        const bool ok = nwritten == 0x200 - rest;
+
+        if (!ok) {
+            setErrorString(
+                tr("Couldn't write alignment: %1")
+                .arg(device()->errorString()));
+        }
+
+        return ok;
     }
     return true;
 }
@@ -678,7 +709,7 @@ void KTar::KTarPrivate::fillBuffer(char *buffer,
     // Header check sum
     int check = 32;
     for (uint j = 0; j < 0x200; ++j) {
-        check += buffer[j];
+        check += static_cast<unsigned char>(buffer[j]);
     }
     s = QByteArray::number(check, 8);   // octal
     s = s.rightJustified(6, '0');
@@ -709,12 +740,14 @@ bool KTar::doPrepareWriting(const QString &name, const QString &user,
                             const QDateTime & /*atime*/, const QDateTime &mtime, const QDateTime & /*ctime*/)
 {
     if (!isOpen()) {
-        //qWarning() << "You must open the tar file before writing to it\n";
+        setErrorString(tr("Application error: TAR file must be open before being written into"));
+        qCWarning(KArchiveLog) << "doPrepareWriting failed: !isOpen()";
         return false;
     }
 
     if (!(mode() & QIODevice::WriteOnly)) {
-        //qWarning() << "You must open the tar file for writing\n";
+        setErrorString(tr("Application error: attempted to write into non-writable 7-Zip file"));
+        qCWarning(KArchiveLog) << "doPrepareWriting failed: !(mode() & QIODevice::WriteOnly)";
         return false;
     }
 
@@ -751,8 +784,8 @@ bool KTar::doPrepareWriting(const QString &name, const QString &user,
     const QByteArray uname = user.toLocal8Bit();
     const QByteArray gname = group.toLocal8Bit();
 
-    // If more than 100 chars, we need to use the LongLink trick
-    if (fileName.length() > 99) {
+    // If more than 100 bytes, we need to use the LongLink trick
+    if (encodedFileName.length() > 99) {
         d->writeLonglink(buffer, encodedFileName, 'L', uname.constData(), gname.constData());
     }
 
@@ -767,7 +800,14 @@ bool KTar::doPrepareWriting(const QString &name, const QString &user,
     d->fillBuffer(buffer, permstr.constData(), size, mtime, 0x30, uname.constData(), gname.constData());
 
     // Write header
-    return device()->write(buffer, 0x200) == 0x200;
+    if (device()->write(buffer, 0x200) != 0x200) {
+        setErrorString(
+            tr("Failed to write header: %1")
+            .arg(device()->errorString()));
+        return false;
+    } else {
+        return true;
+    }
 }
 
 bool KTar::doWriteDir(const QString &name, const QString &user,
@@ -775,12 +815,14 @@ bool KTar::doWriteDir(const QString &name, const QString &user,
                       const QDateTime & /*atime*/, const QDateTime &mtime, const QDateTime & /*ctime*/)
 {
     if (!isOpen()) {
-        //qWarning() << "You must open the tar file before writing to it\n";
+        setErrorString(tr("Application error: TAR file must be open before being written into"));
+        qCWarning(KArchiveLog) << "doWriteDir failed: !isOpen()";
         return false;
     }
 
     if (!(mode() & QIODevice::WriteOnly)) {
-        //qWarning() << "You must open the tar file for writing\n";
+        setErrorString(tr("Application error: attempted to write into non-writable TAR file"));
+        qCWarning(KArchiveLog) << "doWriteDir failed: !(mode() & QIODevice::WriteOnly)";
         return false;
     }
 
@@ -807,8 +849,8 @@ bool KTar::doWriteDir(const QString &name, const QString &user,
     QByteArray uname = user.toLocal8Bit();
     QByteArray gname = group.toLocal8Bit();
 
-    // If more than 100 chars, we need to use the LongLink trick
-    if (dirName.length() > 99) {
+    // If more than 100 bytes, we need to use the LongLink trick
+    if (encodedDirname.length() > 99) {
         d->writeLonglink(buffer, encodedDirname, 'L', uname.constData(), gname.constData());
     }
 
@@ -837,12 +879,14 @@ bool KTar::doWriteSymLink(const QString &name, const QString &target,
                           mode_t perm, const QDateTime & /*atime*/, const QDateTime &mtime, const QDateTime & /*ctime*/)
 {
     if (!isOpen()) {
-        //qWarning() << "You must open the tar file before writing to it\n";
+        setErrorString(tr("Application error: TAR file must be open before being written into"));
+        qCWarning(KArchiveLog) << "doWriteSymLink failed: !isOpen()";
         return false;
     }
 
     if (!(mode() & QIODevice::WriteOnly)) {
-        //qWarning() << "You must open the tar file for writing\n";
+        setErrorString(tr("Application error: attempted to write into non-writable TAR file"));
+        qCWarning(KArchiveLog) << "doWriteSymLink failed: !(mode() & QIODevice::WriteOnly)";
         return false;
     }
 
@@ -861,11 +905,11 @@ bool KTar::doWriteSymLink(const QString &name, const QString &target,
     QByteArray uname = user.toLocal8Bit();
     QByteArray gname = group.toLocal8Bit();
 
-    // If more than 100 chars, we need to use the LongLink trick
-    if (target.length() > 99) {
+    // If more than 100 bytes, we need to use the LongLink trick
+    if (encodedTarget.length() > 99) {
         d->writeLonglink(buffer, encodedTarget, 'K', uname.constData(), gname.constData());
     }
-    if (fileName.length() > 99) {
+    if (encodedFileName.length() > 99) {
         d->writeLonglink(buffer, encodedFileName, 'L', uname.constData(), gname.constData());
     }
 
